"""
This script creates a template file called _pdexplorer.xlsm that
contains all the python code in the pdexplorer package.

When a user runs the insert_pdexplorer script, the code in the 
_pdexplorer.xlsm file will be inserted into user's Excel workbook.

Example usage:
insert_pdexplorer my_excel_sheet.xlsm
"""
import os
import shutil
import sys
import pandas as pd
from openpyxl import load_workbook
from openpyxl.formula.translate import Translator
import click
import xlwings as  xw

file_dir = os.path.dirname(os.path.abspath(__file__))

xlsm_template_file = '_pdexplorer.xlsm'

required_module_names = ["_search.py","_dataset.py","_get_custom_attributes.py","_patsify.py","_print.py",
                    "_commandarg.py","_quietly.py","_print_horizontal_line.py","preserve.py","use.py",
                    "sort.py","_by.py"]

# module_names = ["_search.py","_dataset.py","_get_custom_attributes.py","_patsify.py","_print.py",
#                     "_commandarg.py","_quietly.py","_print_horizontal_line.py","preserve.py","use.py",
#                     "sort.py","_by.py","regress.py","scatter.py","histogram.py","logit.py","drop.py",
#                     "order.py","keep.py"]


exclude = ['make_pdexplorer_xlsm.py', 'merge_pdexplorer_worksheet.py','__init__.py','browse.py']

import os

def list_py_files(file_dir):
    # Get all .py files in the directory
    py_files = [f for f in os.listdir(file_dir) if f.endswith(".py")]
    # Sort the files: those starting with '_' come first
    py_files.sort(key=lambda f: (not f.startswith("_"), f))
    return py_files

# Example usage
# file_dir = "/path/to/your/directory"
all_py_files = list_py_files(file_dir)

for filename in exclude:
    all_py_files.remove(filename)

for filename in required_module_names:
    all_py_files.remove(filename)

module_names = required_module_names + all_py_files + ['__init__.py'] # Add __init__.py at the very end


def make_module_contents_list(module_names, wrap_with_xl_py_formula=True):
    """
    Given a list of module filenames (like ["_search.py"]), read each file's contents
    from the current directory, remove full-line comments (prefixed with "#"),
    then wrap them in a formula string =PY("...").
    Return a list of such strings.
    """
    import os
    module_formulas = []
    for module_name in module_names:
        with open(os.path.join(file_dir, module_name), "r", encoding="utf-8") as f:
            lines = f.readlines()

        processed_lines = []
        # inside_multiline_import = False
        # for line in lines:
        #     stripped_line = line.strip()

        #     # Detect the start of multiline imports
        #     if stripped_line.startswith("from .") or \
        #         stripped_line.startswith("from xlwings") or \
        #         stripped_line.startswith("from pdexplorer.") or \
        #         stripped_line.startswith("hotkey_ctrl_f9"):
        #         inside_multiline_import = True

        #     # If currently inside a multiline import block, check for the end
        #     if inside_multiline_import:
        #         if stripped_line.endswith(")"):  # End of multiline import
        #             inside_multiline_import = False
        #         continue  # Skip the line

        #     # Remove full-line comments
        #     if stripped_line.startswith("#"):
        #         continue

        #     # Special handling for rich import
        #     if stripped_line.startswith("from rich import print as rich_print"):
        #         processed_lines.append("rich_print = print\n")
        #     else:
        #         processed_lines.append(line)
        for line in lines:
            stripped_line = line.strip()
           
            if stripped_line.startswith("#") \
                or stripped_line.startswith("from .") \
                or stripped_line.startswith("from xlwings") \
                or stripped_line.startswith("hotkey_ctrl_f9") \
                or stripped_line.startswith("from pdexplorer."):
                continue # line to skip
            if stripped_line.startswith("from rich import print as rich_print"):
                processed_lines.append("rich_print = print\n")
            else:
                processed_lines.append(line)

        # Join the processed lines
        filtered_code = "".join(processed_lines)

        # Add module name and wrap in =PY
        filtered_code += f'\n\n"{module_name}"'
        if wrap_with_xl_py_formula:
            formula_str = f"""=PY("{filtered_code.replace('"','""')}",1)\n"""
        else:
            formula_str = f"""{filtered_code}\n"""
        module_formulas.append(formula_str)

    return module_formulas

def create_pdexplorer_xlsm(xlsm_template_file):
    wb = xw.Book(xlsm_template_file)
    sheet = wb.sheets[0]
    sheet.clear_contents()
    sheet.range("A1").value = "Module Name"
    sheet.range("B1").value = "Python Object"
    python_cell_list = make_module_contents_list(module_names)
    for i, module_name in enumerate(module_names):
        sheet.range(f"A{i+2}").value = module_name
        sheet.range(f"B{i+2}").formula2 = python_cell_list[i]
    wb.save(xlsm_template_file)
    wb.close()


if __name__ == "__main__":
    create_pdexplorer_xlsm(xlsm_template_file)
    # list(map(make_module_contents_list, module_names))
    # df = pd.DataFrame({"Module": module_names, "Python Object": make_module_contents_list(module_names)})
    # df.to_excel("pdexplorer.xlsm")
    # if os.path.exists("pdexplorer.xlsm"):
    #     os.remove("pdexplorer.xlsm")
    # wb = xw.Book(xlsm_template_file)
    # sheet = wb.sheets[0]
    # sheet.clear_contents()
    # sheet.range("A1").value = "Module Name"
    # sheet.range("B1").value = "Python Object"
    # python_cell_list = make_module_contents_list(module_names)
    # for i, module_name in enumerate(module_names):
    #     sheet.range(f"A{i+2}").value = module_name
    #     sheet.range(f"B{i+2}").formula2 = python_cell_list[i]
    # wb.save(xlsm_template_file)
    # wb.close()
